
#include "VS1053.h"



VS1053Class::VS1053Class(){
}
VS1053Class::~VS1053Class(){
}



int need_data = TRUE; 



/*
****************  Read the 16-bit value of a VS10xx register  ***************
* Private method 
*/
unsigned int VS1053Class::Mp3ReadRegister (unsigned char addressbyte){
 while(!digitalRead(MP3_DREQ)) ; //Wait for DREQ to go high indicating IC is available
 digitalWrite(MP3_XCS, LOW); //Select control

 //SCI consists of instruction byte, address byte, and 16-bit data word.
 SPI.transfer(0x03);  //Read instruction
 SPI.transfer(addressbyte);

 char response1 = SPI.transfer(0xFF); //Read the first byte
 while(!digitalRead(MP3_DREQ)) ; //Wait for DREQ to go high indicating command is complete
 char response2 = SPI.transfer(0xFF); //Read the second byte
 while(!digitalRead(MP3_DREQ)) ; //Wait for DREQ to go high indicating command is complete

 digitalWrite(MP3_XCS, HIGH); //Deselect Control

 int resultvalue = response1 << 8;
 resultvalue |= response2;
 return resultvalue;
}




/*
****************  Write the 16-bit value of a VS10xx register  ***************
* Private method
*/
void VS1053Class::Mp3WriteRegister(unsigned char addressbyte, unsigned char highbyte, unsigned char lowbyte){
 while(!digitalRead(MP3_DREQ)) ; //Wait for DREQ to go high indicating IC is available
 digitalWrite(MP3_XCS, LOW); //Select control

 //SCI consists of instruction byte, address byte, and 16-bit data word.
 SPI.transfer(0x02); //Write instruction
 SPI.transfer(addressbyte);
 SPI.transfer(highbyte);
 SPI.transfer(lowbyte);
 while(!digitalRead(MP3_DREQ)) ; //Wait for DREQ to go high indicating command is complete
 digitalWrite(MP3_XCS, HIGH); //Deselect Control
}




/*
****************  Set VS10xx Volume Register  ***************
* Public method
*/
void VS1053Class::Mp3SetVolume(unsigned char leftchannel, unsigned char rightchannel){
 Mp3WriteRegister(SCI_VOL, leftchannel, rightchannel);
}



/*
****************  Initialize VS1053 chip   ***************
* Public method
*/
void VS1053Class::init(){

 pinMode(MP3_DREQ, INPUT);
 pinMode(MP3_XCS, OUTPUT);
 pinMode(MP3_XDCS, OUTPUT);
 pinMode(MP3_RESET, OUTPUT);
 
 
 pinMode(green_led, OUTPUT);
 pinMode(red_led , OUTPUT);

 

 digitalWrite(MP3_XCS, HIGH); //Deselect Control
 digitalWrite(MP3_XDCS, HIGH); //Deselect Data
 digitalWrite(MP3_RESET, LOW); //Put VS1053 into hardware reset


 digitalWrite(green_led, LOW); 
 digitalWrite(red_led, HIGH); 


 //From page 12 of datasheet, max SCI reads are CLKI/7. Input clock is 12.288MHz. 
 //Internal clock multiplier is 1.0x after power up. 
 //Therefore, max SPI speed is 1.75MHz. We will use 1MHz to be safe.

 SPI.setClockDivider(SPI_CLOCK_DIV16); //Set SPI bus speed to 1MHz (16MHz / 16 = 1MHz)
 SPI.begin();
 SPI.transfer(0xFF); //Throw a dummy byte at the bus


 //Initialize VS1053 chip 
 delay(10);
 digitalWrite(MP3_RESET, HIGH); //Bring up VS1053
 //delay(10); //We don't need this delay because any register changes will check for a high DREQ


 Mp3SetVolume(20, 20); //Set initial volume (20 = -10dB) LOUD
 //Mp3SetVolume(40, 40); //Set initial volume (20 = -10dB) Manageable
 //Mp3SetVolume(80, 80); //Set initial volume (20 = -10dB) More quiet

 //Let's check the status of the VS1053
 int MP3Mode = Mp3ReadRegister(SCI_MODE);
 int MP3Status = Mp3ReadRegister(SCI_STATUS);
 int MP3Clock = Mp3ReadRegister(SCI_CLOCKF);
 int vsVersion = (MP3Status >> 4) & 0x000F; //Mask out only the four version bits
 

 //Now that we have the VS1053 up and running, increase the internal clock multiplier and up our SPI rate
 VS1053Class::Mp3WriteRegister(SCI_CLOCKF, 0x60, 0x00); //Set multiplier to 3.0x

 //From page 12 of datasheet, max SCI reads are CLKI/7. Input clock is 12.288MHz. 
 //Internal clock multiplier is now 3x.
 //Therefore, max SPI speed is 5MHz. 4MHz will be safe.
 SPI.setClockDivider(SPI_CLOCK_DIV4); //Set SPI bus speed to 4MHz (16MHz / 4 = 4MHz)

 MP3Clock = Mp3ReadRegister(SCI_CLOCKF);

 //MP3 IC setup complete

 digitalWrite(green_led, HIGH); 
 digitalWrite(red_led, LOW); 

}



/*
****************  Mp3 buffer status   ***************
* DREQ is low while the receive buffer is full
* Public method
*/
boolean VS1053Class::bufferFull(){
  return !digitalRead(MP3_DREQ);
}


void VS1053Class::playSilence(long ms){
   digitalWrite(MP3_XDCS, LOW); //Select Data
   for (int i = 0 ; i < ( 2*ms) ; i++) {
      while(!digitalRead(MP3_DREQ)); //If we ever see DREQ low, then we wait here
       VS1053.playByte(0);
   }

   digitalWrite(MP3_XDCS, HIGH); //Deselect Data
}

/*
*       ****************  Play one byte   ***************
* Public method
*/
void VS1053Class::playByte(uint8_t  byteToPlay){
  digitalWrite(MP3_XDCS, LOW); //Select Data
  SPI.transfer(byteToPlay);
  digitalWrite(MP3_XDCS, HIGH); //Deselect Data
}


/*
*       ****************  Play buffer   ***************
* Public method
*/
void VS1053Class::playBuffer(uint8_t*  mp3DataBuffer){

 //Once DREQ is released (high) we now feed 32 bytes of data to the VS1053 from our SD read buffer
   digitalWrite(MP3_XDCS, LOW); //Select Data
   for(int y = 0 ; y <  mp3BufferSize ; y++) {
     SPI.transfer(mp3DataBuffer[y]); // Send SPI byte
   }

   digitalWrite(MP3_XDCS, HIGH); //Deselect Data
   need_data = TRUE; //We've just dumped 32 bytes into VS1053 so our SD read buffer is empty. Set flag so we go get more data

}



void substr(char sourceString[],char destString[], int startIndex,int length){
  for(int n=0; n < length; n++)
     destString[n] = sourceString[n+startIndex];
  destString[length] = 0;   
}



void VS1053Class::getMp3Tag(char  mp3TagBuffer[]){



 int tagLength = 200;  
 int n=0;
 int tit2Location =0;
 int tpe1Location =0;
 int tit2Len;
 int tpe1Len; 


   
   
  for(n=0; n <= tagLength; n++ ){
    if(mp3TagBuffer[n] < 32  || mp3TagBuffer[n] > 127  ) continue;
      
    int found =  strncmp(&mp3TagBuffer[n], "TIT2", 4);
    if( tit2Location == 0 &&  found ==0){
       tit2Location = n;
    }
      found =  strncmp(&mp3TagBuffer[n], "TPE1", 4);
      if( tpe1Location == 0 &&  found == 0){
       tpe1Location = n;
    }    
  }
 

tpe1Len = mp3TagBuffer[tpe1Location+7];
tit2Len = mp3TagBuffer[tit2Location+7];

free(mp3TagArtist);
free(mp3TagTitle);

 mp3TagArtist = (char *) malloc(tpe1Len*sizeof(char));
 substr(mp3TagBuffer, mp3TagArtist,tpe1Location+11,tpe1Len-1);

 mp3TagTitle = (char *) malloc(tit2Len*sizeof(char));
 substr(mp3TagBuffer, mp3TagTitle,tit2Location+11,tit2Len-1);

}



